@@ -43,7 +43,6 @@
 #define SETTINGS_KEYBOARD_DIR "org.gnome.settings-daemon.plugins.keyboard"
 
 #define DISABLE_INDICATOR_KEY "disable-indicator"
-#define SHOW_KEYBOARD_LEDS_INDICATOR_KEY "show-keyboard-leds-indicator"
 
 static GsdKeyboardManager *manager = NULL;
 
@@ -65,36 +64,19 @@ static GSettings *settings_keyboard = NULL;
 static PostActivationCallback pa_callback = NULL;
 static void *pa_callback_user_data = NULL;
 
-static const char *gdm_keyboard_layout = NULL;
-
 static GtkStatusIcon *icon = NULL;
 
 static GHashTable *preview_dialogs = NULL;
 
-static Atom caps_lock;
-static Atom num_lock;
-static Atom scroll_lock;
-
-static GtkStatusIcon *indicator_icons[3];
-static const gchar *indicator_on_icon_names[] = {
-	"kbd-scrolllock-on",
-	"kbd-numlock-on",
-	"kbd-capslock-on"
-};
-
-static const gchar *indicator_off_icon_names[] = {
-	"kbd-scrolllock-off",
-	"kbd-numlock-off",
-	"kbd-capslock-off"
-};
-
 static void
 activation_error (void)
 {
 	char const *vendor;
 	GtkWidget *dialog;
 
-	vendor = ServerVendor (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
+	vendor =
+	    ServerVendor (GDK_DISPLAY_XDISPLAY
+			  (gdk_display_get_default ()));
 
 	/* VNC viewers will not work, do not barrage them with warnings */
 	if (NULL != vendor && NULL != strstr (vendor, "VNC"))
@@ -124,8 +106,6 @@ activation_error (void)
 static void
 apply_desktop_settings (void)
 {
-	gboolean show_leds;
-	guint i;
 	if (!inited_ok)
 		return;
 
@@ -134,22 +114,30 @@ apply_desktop_settings (void)
 	/* again, probably it would be nice to compare things
 	   before activating them */
 	gkbd_desktop_config_activate (&current_config);
-
-	show_leds = g_settings_get_boolean (settings_plugin, SHOW_KEYBOARD_LEDS_INDICATOR_KEY);
-	for (i = 0; i < G_N_ELEMENTS (indicator_icons); i++) {
-		gtk_status_icon_set_visible (indicator_icons[i],
-					     show_leds);
-	}
 }
 
 static void
 popup_menu_launch_capplet ()
 {
+	GAppInfo *info;
+	GdkAppLaunchContext *ctx;
 	GError *error = NULL;
 
-	gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
-					  "gnome-keyboard-properties",
-					  &error);
+	info =
+	    g_app_info_create_from_commandline
+	    ("gnome-control-center region", NULL, 0, &error);
+
+	if (info != NULL) {
+		ctx =
+		    gdk_display_get_app_launch_context
+		    (gdk_display_get_default ());
+
+		g_app_info_launch (info, NULL,
+				   G_APP_LAUNCH_CONTEXT (ctx), &error);
+
+		g_object_unref (info);
+		g_object_unref (ctx);
+	}
 
 	if (error != NULL) {
 		g_warning
@@ -293,15 +281,15 @@ show_hide_icon ()
 {
 	if (g_strv_length (current_kbd_config.layouts_variants) > 1) {
 		if (icon == NULL) {
-			gboolean disable =
-			    g_settings_get_boolean (settings_plugin,
-						    DISABLE_INDICATOR_KEY);
-			if (disable)
+			if (g_settings_get_boolean (settings_plugin, DISABLE_INDICATOR_KEY))
+			{
+				xkl_debug (150,
+					   "Not creating keyboard status icon: disabled in GSettings\n");
 				return;
+			}
 
-			xkl_debug (150, "Creating new icon\n");
+			xkl_debug (150, "Creating keyboard status icon\n");
 			icon = gkbd_status_new ();
-			gtk_status_icon_set_name (icon, "keyboard");
 			g_signal_connect (icon, "popup-menu",
 					  G_CALLBACK
 					  (status_icon_popup_menu_cb),
@@ -398,103 +386,10 @@ apply_xkb_settings (void)
 {
 	GkbdKeyboardConfig current_sys_kbd_config;
 	int group_to_activate = -1;
-	char *gdm_layout;
-	char *s;
 
 	if (!inited_ok)
 		return;
 
-	/* With GDM the user can already set a layout from the login
-	 * screen. Try to keep that setting.
-	 * We clear gdm_keyboard_layout early, so we don't risk
-	 * recursion from conf notification.
-	 */
-	gdm_layout = g_strdup (gdm_keyboard_layout);
-	gdm_keyboard_layout = NULL;
-
-	/* gdm's configuration and $GDM_KEYBOARD_LAYOUT separates layout and
-	 * variant with a space, but gsettings use tabs; so convert to be robust
-	 * with both */
-	for (s = gdm_layout; s && *s; ++s) {
-		if (*s == ' ') {
-			*s = '\t';
-		}
-	}
-
-	if (gdm_layout != NULL) {
-		gchar **layouts;
-		gboolean found_node;
-		int max_groups;
-		gchar **i;
-
-		max_groups =
-		    MAX (xkl_engine_get_max_num_groups (xkl_engine), 1);
-		layouts =
-		    g_settings_get_strv (settings_keyboard,
-					 GKBD_KEYBOARD_CONFIG_KEY_LAYOUTS);
-
-		/* Use system layouts as a default if we do not have
-		 * user configuration */
-		if (layouts == NULL) {
-			int len;
-
-			i = layouts =
-			    g_strdupv
-			    (initial_sys_kbd_config.layouts_variants);
-			if (i != NULL) {
-				while (*i != NULL) {
-					s = *i;
-
-					/* chop off empty variants to avoid duplicates */
-					len = strlen (s);
-					if (s[len - 1] == '\t')
-						s[len - 1] = '\0';
-					i++;
-				}
-			}
-		}
-
-		/* Add the layout if it doesn't already exist. XKB limits the
-		 * total number of layouts. If we already have the maximum
-		 * number of layouts configured, we replace the last one. This
-		 * prevents the list from becoming full if the user has a habit
-		 * of selecting many different keyboard layouts in GDM. */
-
-		found_node = FALSE;
-		i = layouts;
-		if (i != NULL) {
-			while (*i != NULL) {
-				if (!g_strcmp0 (*i, gdm_layout)) {
-					found_node = TRUE;
-					break;
-				}
-				i++;
-			}
-		}
-
-		if (!found_node) {
-			/* Insert at the last valid place, or at the end of
-			 * list, whichever comes first */
-			gint old_length = g_strv_length (layouts);
-			if (old_length >= max_groups) {
-				layouts[max_groups - 1] =
-				    g_strdup (gdm_layout);
-			} else {
-				layouts =
-				    gkbd_strv_append (layouts,
-						      g_strdup
-						      (gdm_layout));
-			}
-
-			g_settings_set_strv (settings_keyboard,
-					     GKBD_KEYBOARD_CONFIG_KEY_LAYOUTS,
-					     (const gchar *
-					      const *) layouts);
-		}
-
-		g_strfreev (layouts);
-	}
-
 	gkbd_keyboard_config_init (&current_sys_kbd_config, xkl_engine);
 
 	gkbd_keyboard_config_load (&current_kbd_config,
@@ -520,32 +415,6 @@ apply_xkb_settings (void)
 		xkl_debug (100,
 			   "Actual KBD configuration was not changed: redundant notification\n");
 
-	if (gdm_layout != NULL) {
-		/* If there are multiple layouts,
-		 * try to find the one closest to the gdm layout
-		 */
-		gchar **l;
-		int i;
-		size_t len = strlen (gdm_layout);
-		l = current_kbd_config.layouts_variants;
-		if (l != NULL) {
-			i = 0;
-			while (*l != NULL) {
-				char *lv = *l;
-				if (strncmp (lv, gdm_layout, len) == 0
-				    && (lv[len] == '\0'
-					|| lv[len] == '\t')) {
-					group_to_activate = i;
-					break;
-				}
-				i++;
-				l++;
-			}
-		}
-	}
-
-	g_free (gdm_layout);
-
 	if (group_to_activate != -1)
 		xkl_engine_lock_group (current_config.engine,
 				       group_to_activate);
@@ -588,53 +457,9 @@ gsd_keyboard_new_device (XklEngine * engine)
 	apply_xkb_settings ();
 }
 
-static void
-gsd_keyboard_update_indicator_icons ()
-{
-	Bool state;
-	int new_state;
-	guint i;
-	Display *display =
-	    GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-	XkbGetNamedIndicator (display, caps_lock, NULL, &state, NULL,
-			      NULL);
-	new_state = state ? 1 : 0;
-	XkbGetNamedIndicator (display, num_lock, NULL, &state, NULL, NULL);
-	new_state <<= 1;
-	new_state |= (state ? 1 : 0);
-	XkbGetNamedIndicator (display, scroll_lock, NULL, &state,
-			      NULL, NULL);
-	new_state <<= 1;
-	new_state |= (state ? 1 : 0);
-	xkl_debug (160, "Indicators state: %d\n", new_state);
-
-	for (i = 0; i < G_N_ELEMENTS (indicator_icons); i++) {
-		gtk_status_icon_set_from_icon_name (indicator_icons[i],
-						    (new_state & (1 << i))
-						    ?
-						    indicator_on_icon_names
-						    [i] :
-						    indicator_off_icon_names
-						    [i]);
-	}
-}
-
-static void
-gsd_keyboard_state_changed (XklEngine * engine, XklEngineStateChange type,
-			    gint new_group, gboolean restore)
-{
-	xkl_debug (160,
-		   "State changed: type %d, new group: %d, restore: %d.\n",
-		   type, new_group, restore);
-	if (type == INDICATORS_CHANGED) {
-		gsd_keyboard_update_indicator_icons ();
-	}
-}
-
 void
 gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
 {
-	guint i;
 	Display *display =
 	    GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
 	gnome_settings_profile_start (NULL);
@@ -645,18 +470,6 @@ gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
 					   DATADIR G_DIR_SEPARATOR_S
 					   "icons");
 
-	caps_lock = XInternAtom (display, "Caps Lock", False);
-	num_lock = XInternAtom (display, "Num Lock", False);
-	scroll_lock = XInternAtom (display, "Scroll Lock", False);
-
-	for (i = 0; i < G_N_ELEMENTS (indicator_icons); i++) {
-		indicator_icons[i] =
-		    gtk_status_icon_new_from_icon_name
-		    (indicator_off_icon_names[i]);
-	}
-
-	gsd_keyboard_update_indicator_icons ();
-
 	manager = kbd_manager;
 	gnome_settings_profile_start ("xkl_engine_get_instance");
 	xkl_engine = xkl_engine_get_instance (display);
@@ -664,8 +477,6 @@ gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
 	if (xkl_engine) {
 		inited_ok = TRUE;
 
-		gdm_keyboard_layout = g_getenv ("GDM_KEYBOARD_LAYOUT");
-
 		gkbd_desktop_config_init (&current_config, xkl_engine);
 		gkbd_keyboard_config_init (&current_kbd_config,
 					   xkl_engine);
@@ -688,9 +499,6 @@ gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
 			g_signal_connect (xkl_engine, "X-new-device",
 					  G_CALLBACK
 					  (gsd_keyboard_new_device), NULL);
-		g_signal_connect (xkl_engine, "X-state-changed",
-				  G_CALLBACK
-				  (gsd_keyboard_state_changed), NULL);
 
 		gnome_settings_profile_start ("xkl_engine_start_listen");
 		xkl_engine_start_listen (xkl_engine,
@@ -713,19 +521,13 @@ gsd_keyboard_xkb_init (GsdKeyboardManager * kbd_manager)
 void
 gsd_keyboard_xkb_shutdown (void)
 {
-	guint i;
+	if (!inited_ok)
+		return;
 
 	pa_callback = NULL;
 	pa_callback_user_data = NULL;
 	manager = NULL;
 
-	for (i = 0; i < G_N_ELEMENTS (indicator_icons); i++) {
-		if (indicator_icons[i] != NULL) {
-			g_object_unref (G_OBJECT (indicator_icons[i]));
-			indicator_icons[i] = NULL;
-		}
-	}
-
 	if (preview_dialogs != NULL)
 		g_hash_table_destroy (preview_dialogs);
 
